@@ -226,7 +226,8 @@ write_routes_nm(const NetplanNetDefinition* def, GKeyFile *kf, gint family, GErr
                 || cur_route->congestion_window
                 || cur_route->mtubytes
                 || cur_route->table != NETPLAN_ROUTE_TABLE_UNSPEC
-                || cur_route->from) {
+                || cur_route->from
+                || cur_route->advmss) {
                 tmp_key = g_strdup_printf("route%d_options", j);
                 tmp_val = g_string_new(NULL);
                 if (cur_route->onlink) {
@@ -243,6 +244,8 @@ write_routes_nm(const NetplanNetDefinition* def, GKeyFile *kf, gint family, GErr
                     g_string_append_printf(tmp_val, "table=%u,", cur_route->table);
                 if (cur_route->from)
                     g_string_append_printf(tmp_val, "src=%s,", cur_route->from);
+                if (cur_route->advmss != NETPLAN_ADVMSS_UNSPEC)
+                    g_string_append_printf(tmp_val, "advmss=%u,", cur_route->advmss);
                 tmp_val->str[tmp_val->len - 1] = '\0'; //remove trailing comma
                 g_key_file_set_string(kf, group, tmp_key, tmp_val->str);
                 g_free(tmp_key);
@@ -715,7 +718,7 @@ write_nm_conf_access_point(const NetplanNetDefinition* def, const char* rootdir,
                 return FALSE;
         }
 
-        if (def->type == NETPLAN_DEF_TYPE_VETH) {
+        if (def->type == NETPLAN_DEF_TYPE_VETH && def->veth_peer_link) {
             g_key_file_set_string(kf, "veth", "peer", def->veth_peer_link->id);
         }
     }
@@ -875,7 +878,7 @@ write_nm_conf_access_point(const NetplanNetDefinition* def, const char* rootdir,
         g_key_file_set_boolean(kf, "ipv4", "never-default", TRUE);
     }
 
-    if (def->dhcp4 && def->dhcp4_overrides.metric != NETPLAN_METRIC_UNSPEC)
+    if ((def->dhcp4 || def->ip4_addresses || def->gateway4 || def->routes) && def->dhcp4_overrides.metric != NETPLAN_METRIC_UNSPEC)
         g_key_file_set_uint64(kf, "ipv4", "route-metric", def->dhcp4_overrides.metric);
 
     if (def->dhcp6 || def->ip6_addresses || def->gateway6 || def->ip6_nameservers || def->ip6_addr_gen_mode) {
@@ -1044,10 +1047,12 @@ _netplan_netdef_write_nm(
         GHashTableIter iter;
         gpointer key;
         const NetplanWifiAccessPoint* ap;
-        g_assert(netdef->access_points);
-        g_hash_table_iter_init(&iter, netdef->access_points);
-        while (g_hash_table_iter_next(&iter, &key, (gpointer) &ap) && no_error)
-            no_error = write_nm_conf_access_point(netdef, rootdir, ap, error);
+        if (netdef->access_points) {
+            g_hash_table_iter_init(&iter, netdef->access_points);
+            while (g_hash_table_iter_next(&iter, &key, (gpointer) &ap) && no_error) {
+                no_error = write_nm_conf_access_point(netdef, rootdir, ap, error);
+            }
+        }
     } else {
         g_assert(netdef->access_points == NULL);
         no_error = write_nm_conf_access_point(netdef, rootdir, NULL, error);
