@@ -152,6 +152,7 @@ typedef struct {
     guint mtubytes;
     guint congestion_window;
     guint advertised_receive_window;
+    guint advmss;
 } NetplanIPRoute;
 
 typedef struct {
@@ -263,6 +264,15 @@ struct netplan_parser {
     GHashTable* null_fields;
     GHashTable* null_overrides;
     GHashTable* global_renderer;
+
+    /* Flags used to change the parser's behavior */
+    unsigned int flags;
+
+    /* Number of parsing errors
+     * Records the number of parsing errors that happened
+     * when the flag IGNORE_ERRORS is set
+     * */
+    unsigned int error_count;
 };
 
 struct netplan_state_iterator {
@@ -277,6 +287,7 @@ struct netplan_state_iterator {
 #define NETPLAN_IP_RULE_PRIO_UNSPEC G_MAXUINT
 #define NETPLAN_IP_RULE_FW_MARK_UNSPEC 0
 #define NETPLAN_IP_RULE_TOS_UNSPEC G_MAXUINT
+#define NETPLAN_ADVMSS_UNSPEC 0
 
 #if defined(UNITTESTS)
 #define STATIC
