@@ -18,7 +18,7 @@
 
 import os
 
-from .base import TestBase, ND_DHCP4, UDEV_MAC_RULE, UDEV_NO_MAC_RULE, UDEV_SRIOV_RULE, \
+from .base import TestBase, ND_DHCP4, UDEV_MAC_RULE, UDEV_NO_MAC_RULE, \
     NM_MANAGED, NM_UNMANAGED, NM_MANAGED_MAC, NM_UNMANAGED_MAC, \
     NM_MANAGED_DRIVER, NM_UNMANAGED_DRIVER
 
@@ -82,45 +82,6 @@ LinkLocalAddressing=ipv6
 '''})
         self.assert_networkd_udev(None)
 
-    def test_eth_sriov_vlan_filterv_link(self):
-        self.generate('''network:
-  version: 2
-  ethernets:
-    enp1:
-      dhcp4: n
-    enp1s16f1:
-      dhcp4: n
-      link: enp1''')
-
-        self.assert_networkd({'enp1.network': '''[Match]
-Name=enp1
-
-[Network]
-LinkLocalAddressing=ipv6
-''',
-                              'enp1s16f1.network': '''[Match]
-Name=enp1s16f1
-
-[Network]
-LinkLocalAddressing=ipv6
-'''})
-        self.assert_additional_udev({'99-sriov-netplan-setup.rules': UDEV_SRIOV_RULE})
-
-    def test_eth_sriov_virtual_functions(self):
-        self.generate('''network:
-  version: 2
-  ethernets:
-    enp1:
-      virtual-function-count: 8''')
-
-        self.assert_networkd({'enp1.network': '''[Match]
-Name=enp1
-
-[Network]
-LinkLocalAddressing=ipv6
-'''})
-        self.assert_additional_udev({'99-sriov-netplan-setup.rules': UDEV_SRIOV_RULE})
-
     def test_eth_match_by_driver_rename(self):
         self.generate('''network:
   version: 2
@@ -254,6 +215,46 @@ UseMTU=true
                               })
         self.assert_networkd_udev(None)
 
+    def test_eth_set_mac_value_not_valid(self):
+        res = self.generate('''network:
+  version: 2
+  ethernets:
+    def1:
+      match:
+        name: green
+      macaddress: typorandom
+      dhcp4: true''', expect_fail=True)
+
+        self.assertIn('Invalid MAC address \'typorandom\'', res)
+
+    def test_eth_set_mac_value_only_valid_for_network_manager(self):
+        res = self.generate('''network:
+  version: 2
+  ethernets:
+    def1:
+      match:
+        name: green
+      macaddress: stable
+      dhcp4: true''', expect_fail=True)
+
+        self.assertIn('networkd backend does not support the MAC address option \'stable\'', res)
+
+    def test_eth_set_mac_special_values(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eth0:
+      macaddress: permanent
+      dhcp4: true
+    eth1:
+      macaddress: random
+      dhcp4: true''')
+
+        self.assert_networkd({'eth0.link': '[Match]\nOriginalName=eth0\n\n[Link]\nWakeOnLan=off\nMACAddressPolicy=persistent\n',
+                              'eth0.network': ND_DHCP4 % 'eth0',
+                              'eth1.link': '[Match]\nOriginalName=eth1\n\n[Link]\nWakeOnLan=off\nMACAddressPolicy=random\n',
+                              'eth1.network': ND_DHCP4 % 'eth1'})
+
     def test_eth_match_name_rename(self):
         self.generate('''network:
   version: 2
@@ -384,74 +385,6 @@ method=link-local
 method=ignore
 '''})
 
-    def test_eth_sriov_link(self):
-        self.generate('''network:
-  version: 2
-  renderer: NetworkManager
-  ethernets:
-    enp1:
-      dhcp4: n
-    enp1s16f1:
-      dhcp4: n
-      link: enp1''')
-
-        self.assert_networkd({})
-        self.assert_nm({'enp1': '''[connection]
-id=netplan-enp1
-type=ethernet
-interface-name=enp1
-
-[ethernet]
-wake-on-lan=0
-
-[ipv4]
-method=link-local
-
-[ipv6]
-method=ignore
-''',
-                        'enp1s16f1': '''[connection]
-id=netplan-enp1s16f1
-type=ethernet
-interface-name=enp1s16f1
-
-[ethernet]
-wake-on-lan=0
-
-[ipv4]
-method=link-local
-
-[ipv6]
-method=ignore
-'''})
-        self.assert_additional_udev({'99-sriov-netplan-setup.rules': UDEV_SRIOV_RULE})
-
-    def test_eth_sriov_virtual_functions(self):
-        self.generate('''network:
-  version: 2
-  renderer: NetworkManager
-  ethernets:
-    enp1:
-      dhcp4: n
-      virtual-function-count: 8''')
-
-        self.assert_networkd({})
-        self.assert_nm({'enp1': '''[connection]
-id=netplan-enp1
-type=ethernet
-interface-name=enp1
-
-[ethernet]
-wake-on-lan=0
-
-[ipv4]
-method=link-local
-
-[ipv6]
-method=ignore
-'''})
-        self.assert_additional_udev({'99-sriov-netplan-setup.rules': UDEV_SRIOV_RULE})
-
     def test_eth_set_mac(self):
         self.generate('''network:
   version: 2
@@ -479,6 +412,101 @@ method=auto
 method=ignore
 '''})
 
+    def test_eth_set_mac_special_values(self):
+        self.generate('''network:
+  version: 2
+  renderer: NetworkManager
+  ethernets:
+    eth0:
+      macaddress: preserve
+      dhcp4: true
+    eth1:
+      macaddress: permanent
+      dhcp4: true
+    eth2:
+      macaddress: random
+      dhcp4: true
+    eth3:
+      macaddress: stable
+      dhcp4: true''')
+
+        self.assert_networkd(None)
+
+        self.assert_nm({'eth0': '''[connection]
+id=netplan-eth0
+type=ethernet
+interface-name=eth0
+
+[ethernet]
+wake-on-lan=0
+cloned-mac-address=preserve
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=ignore
+''',
+                       'eth1': '''[connection]
+id=netplan-eth1
+type=ethernet
+interface-name=eth1
+
+[ethernet]
+wake-on-lan=0
+cloned-mac-address=permanent
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=ignore
+''',
+                        'eth2': '''[connection]
+id=netplan-eth2
+type=ethernet
+interface-name=eth2
+
+[ethernet]
+wake-on-lan=0
+cloned-mac-address=random
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=ignore
+''',
+                        'eth3': '''[connection]
+id=netplan-eth3
+type=ethernet
+interface-name=eth3
+
+[ethernet]
+wake-on-lan=0
+cloned-mac-address=stable
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=ignore
+'''})
+
+    def test_eth_set_mac_special_values_error(self):
+        res = self.generate('''network:
+  version: 2
+  renderer: NetworkManager
+  ethernets:
+    eth0:
+      macaddress: preservetypo
+      dhcp4: true''', expect_fail=True)
+
+        error = ("Invalid MAC address 'preservetypo', must be XX:XX:XX:XX:XX:XX, "
+                 "XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX:XX or "
+                 "one of 'permanent', 'random', 'stable', 'preserve'")
+        self.assertIn(error, res)
+
     def test_eth_match_by_driver(self):
         err = self.generate('''network:
   version: 2
