@@ -835,6 +835,54 @@ Gateway=192.168.14.20
 Metric=4294967294
 '''})
 
+    def test_route_v4_advmss_systemd(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: ["192.168.14.2/24"]
+      routes:
+        - to: 10.10.10.0/24
+          via: 192.168.14.20
+          advertised-mss: 1400
+          ''')
+
+        self.assert_networkd({'engreen.network': '''[Match]
+Name=engreen
+
+[Network]
+LinkLocalAddressing=ipv6
+Address=192.168.14.2/24
+
+[Route]
+Destination=10.10.10.0/24
+Gateway=192.168.14.20
+TCPAdvertisedMaximumSegmentSize=1400
+'''})
+
+    def test_route_v4_advmss_empty_systemd(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: ["192.168.14.2/24"]
+      routes:
+        - to: 10.10.10.0/24
+          via: 192.168.14.20
+          ''')
+
+        self.assert_networkd({'engreen.network': '''[Match]
+Name=engreen
+
+[Network]
+LinkLocalAddressing=ipv6
+Address=192.168.14.2/24
+
+[Route]
+Destination=10.10.10.0/24
+Gateway=192.168.14.20
+'''})
+
 
 class TestNetworkManager(TestBase):
 
@@ -1610,6 +1658,37 @@ method=manual
 address1=192.168.14.2/24
 route1=10.10.10.0/24,192.168.14.20,4294967294
 
+[ipv6]
+method=ignore
+'''})
+
+    def test_route_v4_advmss_nm(self):
+        self.generate('''network:
+  version: 2
+  renderer: NetworkManager
+  ethernets:
+    engreen:
+      addresses: ["192.168.14.2/24"]
+      routes:
+        - to: 10.10.10.0/24
+          via: 192.168.14.20
+          advertised-mss: 1400
+          ''')
+
+        self.assert_nm({'engreen': '''[connection]
+id=netplan-engreen
+type=ethernet
+interface-name=engreen
+
+[ethernet]
+wake-on-lan=0
+
+[ipv4]
+method=manual
+address1=192.168.14.2/24
+route1=10.10.10.0/24,192.168.14.20
+route1_options=advmss=1400
+
 [ipv6]
 method=ignore
 '''})
