@@ -24,12 +24,20 @@
 import sys
 import subprocess
 import unittest
+import gi
 
 from base import IntegrationTestsBase, test_backends
+from packaging.version import Version
 
 
 class _CommonTests():
 
+    def _nm_version(self):
+        gi.require_version('NM', '1.0')
+        from gi.repository import NM
+        nm_client = NM.Client.new(None)
+        return Version(nm_client.get_version())
+
     # Supposed to fail if tested against NetworkManager < 1.12/1.18
     # The on-link option was introduced as of NM 1.12+ (for IPv4)
     # The on-link option was introduced as of NM 1.18+ (for IPv6)
@@ -278,6 +286,11 @@ class _CommonTests():
                       subprocess.check_output(['ip', 'route', 'show', '10.10.10.0/24']))
 
     def test_vrf_basic(self):
+        try:
+            subprocess.check_call(['modprobe', 'vrf'])
+        except Exception:
+            raise unittest.SkipTest("vrf module is unavailable, can't test")
+
         self.setup_eth('slaac')
         self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'vrf0'], stderr=subprocess.DEVNULL)
         self.addCleanup(subprocess.call, ['ip', 'route', 'flush', 'table', '1000'], stderr=subprocess.DEVNULL)
@@ -323,6 +336,56 @@ class _CommonTests():
             out = subprocess.check_output(['ip', 'rule', 'show'], text=True)
             self.assertIn('from 10.10.10.42 lookup 1000', out)
 
+    def test_route_advmss_v6(self):
+        self.setup_eth(None)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+    renderer: %(r)s
+    ethernets:
+      ethbn:
+        match: {name: %(ec)s}
+        addresses: ["9876:BBBB::11/70"]
+        routes:
+          - to: 2001:f00f:f00f::1/64
+            via: 9876:BBBB::5
+            advertised-mss: 1400''' % {'r': self.backend, 'ec': self.dev_e_client})
+        self.generate_and_settle([self.dev_e_client])
+
+        # wait for NetworkManager to be started (by generate_and_settle()),
+        # before checking its version.
+        if self.backend == 'NetworkManager' and self._nm_version() < Version('1.39.8'):
+            # https://github.com/NetworkManager/NetworkManager/commit/90e7afc
+            raise unittest.SkipTest('advmss is not yet supported in this version of NetworkManager')
+
+        self.assert_iface_up(self.dev_e_client, ['inet6 9876:bbbb::11/70'])
+        out = subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client], text=True)
+        self.assertRegex(out, r'2001:f00f:f00f::/64 via 9876:bbbb::5 proto static[^\n]* advmss 1400')
+
+    def test_route_advmss_v4(self):
+        self.setup_eth(None)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+    renderer: %(r)s
+    ethernets:
+      ethbn:
+        match: {name: %(ec)s}
+        addresses: ["192.168.1.1/24"]
+        routes:
+          - to: 172.29.29.0/24
+            via: 192.168.1.254
+            advertised-mss: 1350''' % {'r': self.backend, 'ec': self.dev_e_client})
+        self.generate_and_settle([self.dev_e_client])
+
+        # wait for NetworkManager to be started (by generate_and_settle()),
+        # before checking its version.
+        if self.backend == 'NetworkManager' and self._nm_version() < Version('1.39.8'):
+            # https://github.com/NetworkManager/NetworkManager/commit/90e7afc
+            raise unittest.SkipTest('advmss is not yet supported in this version of NetworkManager')
+
+        self.assert_iface_up(self.dev_e_client, ['inet 192.168.1.1/24'])
+        out = subprocess.check_output(['ip', 'route', 'show', 'dev', self.dev_e_client], text=True)
+        self.assertRegex(out, r'172.29.29.0/24 via 192.168.1.254 proto static[^\n]* advmss 1350')
+
 
 @unittest.skipIf("networkd" not in test_backends,
                  "skipping as networkd backend tests are disabled")
